# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/3/20] ------------------------- #
# --------------------------------------------- #
module Strum::Internal

  module Component
    include Container
    ##
    # Each component has it's own Reactor(executor) for async tasks.
    def initialize(type)
      self[:type] = type
      case type
      when :Container
        self[:Executor] = Async::Container::Controller.new
        report "Spawned #{class_name} executor!"
      when :Reactor
        self[:Executor] = Async::Reactor.new
        report "Spawned #{class_name} executor!"
      else
        self[:Executor] = Async::Reactor.new
        report "No component_type specified! Defaulted executor to Reactor type for #{class_name}!"
      end
    end

    ##
    # Executes an Asynchronous task.
    def asynchronously(&block)
      raise Strum::Errors::NilAsyncBlock unless block_given?

      task = Async::Task.current
      task.sleep(0.0005)
      task.async { block.call(self) }
    end

    ##
    # Puts the current Task to sleep for specified time (Poor name?)
    # @param duration [Numeric] - How long should this task sleep for?
    def tranquilize(duration)
      Async::Task.current.sleep(duration)
    end

    ##
    # Hooks a receiver object to this component.
    # @param type [Symbol] the type of object
    # @param receiver [Object] the receiver we're hooking.
    def hook(type, receiver)
      self[type] = receiver
    end

    ##
    # Allow the children to use it.
    # @param lines [Array] lines of text that will be reported by the logger.
    def report(*lines)
      lines.each { |line| Strum::LOG.info "[#{self.class.name.split('::').last.to_sym}] 🢐(#{line})" }
      nil
    end

    private

    ##
    # Executes a task using this component's executor.
    def execute_task(&block)
      raise Strum::Errors::NilBlockInvocation.new unless block_given?

      self[:Executor].run { Async { |task|  yield(task) } }
      self[:Executor].wait if self[:type] == :Container
      self[:Executor].stop if self[:Executor]
    end
  end
end